/*
    multitimerview.cpp  -  The main view widget

    Copyright (c) 2005      by Michal Larouche       <michael.larouche@kdemail.net>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#include "multitimerview.h"

#include "timertabpage.h"
#include "timercontrolwidget.h"

#include <QVector>
#include <QMap>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QSplitter>
#include <QSettings>
#include <QMessageBox>
#include <QGroupBox>

class AllTab : public QWidget
{
public:
	AllTab(QWidget *parent=0)
		: QWidget(parent)
	{
		layout = new QVBoxLayout(this);
	}

	void addWidget(QWidget *widget)
	{
		layout->addWidget(widget);
	}

private:
	QVBoxLayout *layout;
};

class MultiTimerView::Private
{
public:
	Private() : numTimers(5), allTab(0L)
	{}

	QVector<TimerTabPage*> tabList;
	QVector<TimerControlWidget*> timerControlList;
	int numTimers;
	AllTab *allTab;
};

MultiTimerView::MultiTimerView(QWidget *parent)
: QTabWidget(parent)
{
	d = new Private;
}

MultiTimerView::~MultiTimerView()
{
	delete d;
}

void MultiTimerView::addTimer()
{
	int newIndex = count();

	// Creat the Timer Control Widget associated with the timer.
	TimerControlWidget *controlWidget = new TimerControlWidget(this, newIndex);
	d->allTab->addWidget(controlWidget);
	d->timerControlList.append(controlWidget);

	// Craete the Timer Tab page.
	TimerTabPage *tabPage = createSingleTabPage(newIndex);
	QSettings stub;
	tabPage->readSettings(stub);// Read the default values.
	d->tabList.append(tabPage);

	addTab(tabPage, "Timer");
	tabIdChanged(newIndex, tr("Timer %1").arg(newIndex));

	d->numTimers++;
}

void MultiTimerView::removeTimer()
{
	int removedIndex = currentIndex() -1;

	if(removedIndex < 0) // Don't allow to remove the all tab.
	{
		QMessageBox::warning(this, tr("Warning"), tr("You can't remove the \"All\" tab !"), QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);
		return;
	}

	int result;
	result = QMessageBox::question(this, tr("Warning"), tr("Do you really want to remove this timer tab?"), QMessageBox::Yes, QMessageBox::No);
	if(result == QMessageBox::Yes)
	{
		TimerTabPage *removedTabPage = d->tabList[removedIndex];

		// Clean up the settings.
		QSettings settings(QSettings::UserScope, "shockdev.ca.tc", "MultiTimerQt"); 
		settings.beginGroup(QString("Timer %1").arg(removedIndex+1));
		removedTabPage->removeSettings(settings);
		settings.endGroup();

		// Remove the tab from the vector and delete it.
		d->tabList.remove(removedIndex);
		removedTabPage->deleteLater();

		// Remove the Timer Control Widget
		TimerControlWidget *removedControlWidget = d->timerControlList[removedIndex];
		d->timerControlList.remove(removedIndex);
		removedControlWidget->deleteLater();

		// Subcontrols index maybe are all fucked up. Reset the indexes;
		int controlIndex = 1;
		QVector<TimerControlWidget*>::iterator controlIt;
		for(controlIt = d->timerControlList.begin(); controlIt != d->timerControlList.end(); ++controlIt)
		{
			TimerControlWidget *controlWidget = *controlIt;
			controlWidget->setIndex(controlIndex);
			controlIndex++;
		}
		int tabIndex = 1;
		QVector<TimerTabPage*>::iterator tabIt;
		for(tabIt = d->tabList.begin(); tabIt != d->tabList.end(); ++tabIt)
		{
			TimerTabPage *tabPage = *tabIt;
			tabPage->setIndex(tabIndex);
			tabIndex++;
		}
		d->numTimers--;

		setCurrentIndex(removedIndex);
	}
}

void MultiTimerView::readSettings()
{
	QSettings settings(QSettings::UserScope, "shockdev.ca.tc", "MultiTimerQt"); 
	d->numTimers = settings.value("view/numtimers", 5).toInt();
	createAllTabPage();
	createAllTimerPage();
}

void MultiTimerView::writeSettings()
{
	QSettings settings(QSettings::UserScope, "shockdev.ca.tc", "MultiTimerQt"); 
	settings.setValue("view/numtimers", d->numTimers);

	QVector<TimerTabPage*>::iterator it;
	for(it = d->tabList.begin(); it != d->tabList.end(); ++it)
	{
		TimerTabPage *tabPage = *it;
		settings.beginGroup(QString("Timer %1").arg(tabPage->index()));
		tabPage->writeSettings(settings);
		settings.endGroup();
	}
}

bool MultiTimerView::isTimerActive() const
{
	bool timerActive=false;
	QVector<TimerTabPage*>::iterator it;
	for(it = d->tabList.begin(); it != d->tabList.end(); ++it)
	{
		TimerTabPage *tabPage = *it;
		if(tabPage->isStarted())
		{
				timerActive = true;
				break;
		}
	}

	return timerActive;
}

void MultiTimerView::createAllTabPage()
{
	d->allTab = new AllTab(this);

	for(int i=1;i<=d->numTimers;i++)
	{
		TimerControlWidget *controlWidget = new TimerControlWidget(this, i);
		d->allTab->addWidget(controlWidget);
		d->timerControlList.append(controlWidget);
	}

	addTab(d->allTab, tr("All"));
}

void MultiTimerView::createAllTimerPage()
{
	for(int i=1;i<=d->numTimers;i++)
	{
		QSettings settings(QSettings::UserScope, "shockdev.ca.tc", "MultiTimerQt"); 
		TimerTabPage *tabPage = createSingleTabPage(i);
		
		// Reset the previous settings.
		settings.beginGroup(QString("Timer %1").arg(i));
		tabPage->readSettings(settings);
		
		d->tabList.append(tabPage);
		addTab(tabPage, "Timer");
		tabIdChanged(i, settings.value("timerId", tr("Timer %1").arg(i)).toString());
		settings.endGroup();
	}
}

TimerTabPage* MultiTimerView::createSingleTabPage(int index)
{
	TimerTabPage *tabPage = new TimerTabPage(this, index);
	
	connect(tabPage, SIGNAL(timerTick(const QTime &)), d->timerControlList[index-1], SLOT(updateTimeEdit(const QTime &)));
	connect(d->timerControlList[index-1], SIGNAL(buttonClicked()), tabPage, SLOT(slotStartTimer()));
	connect(tabPage, SIGNAL(timerStateChanged()), d->timerControlList[index-1], SLOT(toggleButton()));
	connect(tabPage, SIGNAL(idChanged(int, const QString &)), this, SLOT(tabIdChanged(int, const QString &)));

	return tabPage;
}

void MultiTimerView::tabIdChanged(int index, const QString &text)
{
	setTabText(index, text);
	d->timerControlList[index -1]->updateText(text);
}
