/*
    timertabpage.cpp -  Widget for a single tab in tab widget.

    Copyright (c) 2005      by Michal Larouche       <michael.larouche@kdemail.net>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#include "timertabpage.h"

#include "digitaltimewidget.h"
#include "alarmdialog.h"

#include <QPushButton>
#include <QTimeEdit>
#include <QLineEdit>
#include <QCheckBox>
#include <QRadioButton>
#include <QTimer>
#include <QTime>
#include <QSettings>

#include <QMessageBox>

class TimerTabPage::Private
{
public:
	Private() :
	  digitalTime(0L), time1(), timerUpdate(0L), timerCountDown(0L), isCountDown(false),isPaused(false)
	  {}

	DigitalTimeWidget *digitalTime;
	QTime time1;
	QTimer *timerUpdate;
	QTimer *timerCountDown;
	bool isCountDown;
	bool isPaused;
	int index;
};

TimerTabPage::TimerTabPage(QWidget *parent, int index)
: QWidget(parent)
{
	d = new Private;
	d->index = index;

	setupUi(this);

	d->digitalTime = new DigitalTimeWidget(this);
	vboxLayout->addWidget(d->digitalTime);

	connect(buttonStart, SIGNAL(clicked()), this, SLOT(slotStartTimer()));
	connect(buttonPause, SIGNAL(clicked()), this, SLOT(slotPauseTimer()));

	connect(lineId, SIGNAL(textChanged(const QString &)), this, SLOT(slotIdChanged(const QString &)));
	connect(timeEditTimer, SIGNAL(timeChanged(const QTime &)), this, SIGNAL(timerTick(const QTime &)));
	connect(timeEditTimer, SIGNAL(timeChanged(const QTime &)), this, SLOT(slotDisplayTimeChange()));
}

TimerTabPage::~TimerTabPage()
{
	delete d;
}

void TimerTabPage::readSettings(QSettings &settings)
{
	lineId->setText(settings.value("timerId", tr("Timer %1").arg(d->index)).toString());
	lineMessage->setText(settings.value("timerMessage", "").toString());
	timeEditTimer->setTime(settings.value("timerTime", QTime()).toTime());
	checkSound->setChecked(settings.value("timerSound", false).toBool());
	radioCountDown->setChecked(settings.value("timerCountDown", false).toBool());
	radioCountUp->setChecked(settings.value("timerCountUp", true).toBool());
}

void TimerTabPage::removeSettings(QSettings &settings)
{
	settings.remove("timerId");
	settings.remove("timerMessage");
	settings.remove("timerTime");
	settings.remove("timerSound");
	settings.remove("timerCountDown");
	settings.remove("timerCountUp");
}

void TimerTabPage::writeSettings(QSettings &settings)
{
	settings.setValue("timerId", lineId->text());
	settings.setValue("timerMessage", lineMessage->text());
	settings.setValue("timerTime", timeEditTimer->time());
	settings.setValue("timerSound", checkSound->isChecked());
	settings.setValue("timerCountDown", radioCountDown->isChecked());
	settings.setValue("timerCountUp", radioCountUp->isChecked());
}

int TimerTabPage::index() const
{
	return d->index;
}

void TimerTabPage::setIndex(int index)
{
	d->index = index;
}

bool TimerTabPage::isStarted()
{
	return (d->timerUpdate != 0L);
}

void TimerTabPage::slotDisplayTicks()
{
	d->digitalTime->showTime(d->time1);
	emit timerTick(d->time1);
}

void TimerTabPage::slotDisplayTimeChange()
{
	d->digitalTime->showTime(timeEditTimer->time());
}

void TimerTabPage::slotUpdateTimer()
{
	if(d->isCountDown)
	{
		d->time1 = d->time1.addSecs(-1);
	}
	else
	{
		d->time1 = d->time1.addSecs(1);
	}
}

void TimerTabPage::slotStartTimer()
{
	// If we have a timer, then the timer is active, stop the timer.
	if(d->timerUpdate != 0L || d->timerCountDown != 0L)
	{
		d->timerUpdate->stop();
		d->timerUpdate->deleteLater();
		d->timerUpdate = 0L;
		
		if(d->timerCountDown != 0L)
		{
			d->timerCountDown->stop();
			d->timerCountDown->deleteLater();
			d->timerCountDown = 0L;
			d->isCountDown = false;
		}

		d->time1 = radioCountDown->isChecked() ? timeEditTimer->time() : QTime(0,0,0,0);
		slotDisplayTicks();

		buttonStart->setText(tr("START"));
		buttonPause->setText(tr("PAUSE"));
		slotEnableAndDisableWidgets(true);
	}
	else
	{
		d->timerUpdate = new QTimer(this);
		d->timerUpdate->setInterval(1000);
		connect(d->timerUpdate, SIGNAL(timeout()), this, SLOT(slotDisplayTicks()));
		connect(d->timerUpdate, SIGNAL(timeout()), this, SLOT(slotUpdateTimer()));
		d->isCountDown = false;

		if(radioCountDown->isChecked())
		{	
			int msecs = getTotalMSecsFromTime(timeEditTimer->time());
			d->time1 = QTime(timeEditTimer->time());
			d->isCountDown = true;

			d->timerCountDown = new QTimer(this);
			d->timerCountDown->setInterval(msecs+1000);
			connect(d->timerCountDown, SIGNAL(timeout()), this, SLOT(slotShowAlarm()));
			//d->timerCountDown->start(msecs+1000);
			d->timerCountDown->start();
		}
		
		//d->timerUpdate->start(1000);
		d->timerUpdate->start();

		buttonStart->setText(tr("STOP"));
		slotEnableAndDisableWidgets(false);
	}
	emit timerStateChanged();
}

void TimerTabPage::slotShowAlarm()
{
	slotStartTimer(); // Stop the timer and reset the display.
	AlarmDialog *alarmDialog = new AlarmDialog(this, lineId->text(), lineMessage->text(), checkSound->isChecked());
	alarmDialog->setWindowTitle(tr("Alarm: %1 - %2").arg(lineId->text()).arg(timeEditTimer->time().toString("hh:mm:ss")));
	alarmDialog->setModal(true);
	connect(alarmDialog, SIGNAL(restartTimer()), this, SLOT(slotStartTimer()));
	alarmDialog->show();
}

void TimerTabPage::slotPauseTimer()
{
	if(d->isPaused)
	{
		d->isPaused = false;
		// Restart the countdown timer with the msecs that last to be done.
		if(d->isCountDown)
			d->timerCountDown->start(getTotalMSecsFromTime(d->time1)+1000);
		d->timerUpdate->start();
		
		buttonPause->setText(tr("PAUSE"));
		
	}
	else
	{
		d->isPaused = true;
		d->timerUpdate->stop();
		if(d->isCountDown)
			d->timerCountDown->stop();
		
		buttonPause->setText(tr("RESUME"));
	}
}

void TimerTabPage::slotEnableAndDisableWidgets(bool enable)
{
	timeEditTimer->setEnabled(enable);
	checkSound->setEnabled(enable);
	radioCountDown->setEnabled(enable);
	radioCountUp->setEnabled(enable);
	lineMessage->setEnabled(enable);
	lineId->setEnabled(enable);
	buttonPause->setEnabled(!enable); // If the other are true, then the pause button is not enabled.
}

void TimerTabPage::slotIdChanged(const QString &text)
{
	emit idChanged(d->index, text);
}

int TimerTabPage::getTotalMSecsFromTime(const QTime &time)
{
	QTime zeroTime(0,0,0,0);
	return zeroTime.msecsTo(time);
}
